@@ -21,6 +21,7 @@ from .attachment import Attachment
 import copy
 import base64
 import os
+import pathlib
 
 
 class AttrHolder:
@@ -114,7 +115,7 @@ class AttrHolder:
             elif attr.endswith("Date"):
                 self.__dict__["_" + attr] = format_timestamp(data.get(attr))
 
-            elif attr in ["registrator", "modifier", "dataProducer"]:
+            elif attr in ["registrator", "modifier", "dataProducer", "owner"]:
                 self.__dict__["_" + attr] = extract_person(data.get(attr))
 
             else:
@@ -256,9 +257,7 @@ class AttrHolder:
                                     "@type": "as.dto.person.id.PersonPermId",
                                 }
                             ],
-                            "@type": "as.dto.common.update.ListUpdateAction{}".format(
-                                self.__dict__["_changed_users"][userId]["action"]
-                            ),
+                            "@type": f'as.dto.common.update.ListUpdateAction{self.__dict__["_changed_users"][userId]["action"]}',
                         }
                     )
 
@@ -285,8 +284,9 @@ class AttrHolder:
                 # all items in a list
                 if "multi" in self._defs and attr in self._defs["multi"]:
                     items = self.__dict__.get("_" + attr, [])
-                    if items == None:
-                        items = []
+                    if items is None:
+                        # 'None' multivalue attributes should be omitted during update
+                        continue
                     up_obj[attr2ids[attr]] = {
                         "actions": [
                             {
@@ -452,16 +452,12 @@ class AttrHolder:
         if self._is_new:
             if name not in self._defs["attrs_new"]:
                 raise ValueError(
-                    "No such attribute: «{}» for entity: {}. Allowed attributes are: {}".format(
-                        name, self.entity, self._defs["attrs_new"]
-                    )
+                    f'No such attribute: «{name}» for entity: {self.entity}. Allowed attributes are: {self._defs["attrs_new"]}'
                 )
         else:
             if name not in self._defs["attrs_up"]:
                 raise ValueError(
-                    "No such attribute: «{}» for entity: {}. Allowed attributes are: {}".format(
-                        name, self.entity, self._defs["attrs_up"]
-                    )
+                    f'No such attribute: «{name}» for entity: {self.entity}. Allowed attributes are: {self._defs["attrs_up"]}'
                 )
 
         if name in ["parents", "children", "components"]:
@@ -578,14 +574,12 @@ class AttrHolder:
                 self.__dict__["_" + name]["isModified"] = True
 
         elif name in ["identifier"]:
-            raise KeyError("you can not modify the {}".format(name))
+            raise KeyError(f"you can not modify the {name}")
         elif name == "code":
             try:
                 if self._type["autoGeneratedCode"]:
                     raise KeyError(
-                        "This {}Type has auto-generated code. You cannot set a code".format(
-                            self.entity
-                        )
+                        f"This {self.entity}Type has auto-generated code. You cannot set a code"
                     )
             except KeyError:
                 pass
@@ -605,7 +599,7 @@ class AttrHolder:
             # enum: check whether value is a valid constant
             if value and value not in self._defs[name]:
                 raise ValueError(
-                    "Allowed values for enum {} are: {}".format(name, self._defs[name])
+                    f"Allowed values for enum {name} are: {self._defs[name]}"
                 )
             else:
                 self.__dict__["_" + name] = value
@@ -752,6 +746,8 @@ class AttrHolder:
         """add parent to _parents list"""
         if not isinstance(parents_to_add, list):
             parents_to_add = [parents_to_add]
+        if self.__dict__["_parents"] is None:
+            self.__dict__["_parents"] = []
         for parent in parents_to_add:
             ident = self._ident_for_whatever(parent)
             if ident not in self.__dict__["_parents"]:
@@ -761,6 +757,8 @@ class AttrHolder:
         """remove parent from _parents list"""
         if not isinstance(parents_to_remove, list):
             parents_to_remove = [parents_to_remove]
+        if self.__dict__["_parents"] is None:
+            self.__dict__["_parents"] = []
         for parent in parents_to_remove:
             ident = self._ident_for_whatever(parent)
             for i, item in enumerate(self.__dict__["_parents"]):
@@ -794,6 +792,8 @@ class AttrHolder:
         """add children to _children list"""
         if getattr(self, "_children") is None:
             self.__dict__["_children"] = []
+        if self.__dict__["_children"] is None:
+            self.__dict__["_children"] = []
         if not isinstance(children, list):
             children = [children]
         for child in children:
@@ -805,6 +805,8 @@ class AttrHolder:
             return
         if not isinstance(children, list):
             children = [children]
+        if self.__dict__["_children"] is None:
+            self.__dict__["_children"] = []
         for child in children:
             ident = self._ident_for_whatever(child)
             for i, item in enumerate(self.__dict__["_children"]):
@@ -923,32 +925,35 @@ class AttrHolder:
             self.__dict__["_new_attachments"] = []
         self._new_attachments.append(att)
 
-    def download_attachments(self):
-        method = "get" + self.entity + "s"
-        entity = self.entity.lower()
+    def download_attachments(self, destination_folder=None):
+        method = "get" + self.entity.lower().capitalize() + "s"
         request = {
             "method": method,
             "params": [
                 self._openbis.token,
                 [self._permId],
-                dict(
-                    attachments=fetch_option["attachmentsWithContent"],
-                    **fetch_option[entity]
-                ),
+                {
+                    **fetch_option[self.entity.lower()],
+                    "attachments": fetch_option["attachmentsWithContent"],
+                },
             ],
         }
         resp = self._openbis._post_request(self._openbis.as_v3, request)
         attachments = resp[self.permId]["attachments"]
         file_list = []
+
+        if destination_folder is None:
+            destination_folder = os.getcwd()
         for attachment in attachments:
-            filename = os.path.join(
-                self._openbis.hostname, self.permId, attachment["fileName"]
+            filename = (
+                pathlib.Path(destination_folder)
+                / pathlib.Path(attachment["fileName"]).name
             )
             os.makedirs(os.path.dirname(filename), exist_ok=True)
             with open(filename, "wb") as att:
                 content = base64.b64decode(attachment["content"])
                 att.write(content)
-            file_list.append(filename)
+            file_list.append(str(filename))
         return file_list
 
     def all(self):
@@ -981,9 +986,7 @@ class AttrHolder:
         for attr in attrs:
             if attr == "attachments":
                 continue
-            html += "<tr> <td>{}</td> <td>{}</td> </tr>".format(
-                attr, nvl(getattr(self, attr, ""), "")
-            )
+            html += f'<tr> <td>{attr}</td> <td>{nvl(getattr(self, attr, ""), "")}</td> </tr>'
         if getattr(self, "_attachments") is not None:
             html += "<tr><td>attachments</td><td>"
             html += "<br/>".join(att["fileName"] for att in self._attachments)
@@ -1013,9 +1016,7 @@ class AttrHolder:
                 for role in self._roleAssignments:
                     if role.get("space") is not None:
                         roles.append(
-                            "{} ({})".format(
-                                role.get("role"), role.get("space").get("code")
-                            )
+                            f'{role.get("role")} ({role.get("space").get("code")})'
                         )
                     else:
                         roles.append(role.get("role"))
